import gevent

from common.status import WorkerStatus
from worker import Worker


class ScaleHub(object):
    def __init__(self, config):
        """
        type config: dict
        """
        self.workers_config = config['workers']
        self.workers = []
        for worker_config in self.workers_config:
            self.workers.append(Worker(worker_config))

    def go(self):
        self.run_workers()
        self.start_test()

    def is_test_completed(self):
        return reduce(
            lambda prev, next: prev and (next.status() == WorkerStatus.test_completed),  # noqa
            self.workers,
            True
        )

    def run_workers(self):
        threads = [gevent.spawn(worker.run) for worker in self.workers]
        gevent.joinall(threads)

        running_workers = range(0, len(self.workers))
        while len(running_workers) > 0:
            for i in running_workers:
                if self.workers[i].status() == WorkerStatus.init_completed:
                    running_workers.remove(i)

    def start_test(self):
        threads = [gevent.spawn(worker.start_test) for worker in self.workers]
        gevent.joinall(threads)
